﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  [Item("SolutionCachingAnalyzer", "An operator that records solutions.")]
  [StorableClass]
  public class SolutionCachingAnalyzer : SingleSuccessorOperator, IAnalyzer {
    private const string ResultsParameterName = "Results";
    //TODO: this should be a stateful item!
    #region IAnalyzer Members
    public bool EnabledByDefault {
      get { return false; }
    }
    #endregion

    #region Parameter properties
    public ILookupParameter<ResultCollection> ResultsParameter {
      get { return (ILookupParameter<ResultCollection>)Parameters[ResultsParameterName]; }
    }
    public ILookupParameter<IntValue> GenerationsParameter {
      get { return (ILookupParameter<IntValue>)Parameters["Generations"]; }
    }
    public ScopeTreeLookupParameter<Permutation> SolutionParameter {
      get { return (ScopeTreeLookupParameter<Permutation>)Parameters["Solution"]; }
    }
    #endregion

    #region Properties
    public ResultCollection Results {
      get { return ResultsParameter.ActualValue; }
    }

    [Storable]
    private PermutationSolutionDictionary<PermutationWrapper, int> solDict;
    #endregion

    [StorableConstructor]
    private SolutionCachingAnalyzer(bool deserializing) : base(deserializing) { }

    private SolutionCachingAnalyzer(SolutionCachingAnalyzer original, Cloner cloner)
      : base(original, cloner) {
      this.solDict = (PermutationSolutionDictionary<PermutationWrapper, int>)original.solDict.Clone(cloner);
    }

    public SolutionCachingAnalyzer()
      : base() {
      Parameters.Add(new LookupParameter<ResultCollection>(ResultsParameterName, "The results collection where the analysis values should be stored."));
      Parameters.Add(new LookupParameter<IntValue>("Generations", "Nr of generations."));
      Parameters.Add(new ScopeTreeLookupParameter<Permutation>("Solution", "The solutions that should be stored.", "TSPTour", 1));
      solDict = new PermutationSolutionDictionary<PermutationWrapper, int>();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SolutionCachingAnalyzer(this, cloner);
    }

    public override IOperation Apply() {
      ItemArray<Permutation> solutions = SolutionParameter.ActualValue;

      foreach (var sol in solutions) {
        solDict.Add(new PermutationWrapper(sol), GenerationsParameter.ActualValue.Value);
      }

      if (!ResultsParameter.ActualValue.ContainsKey("SolutionCache")) {
        ResultsParameter.ActualValue.Add(new Result("SolutionCache", solDict));
      }

      return base.Apply();
    }
  }
}
